"""
(c) 2017 DigitalOcean

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from pynetbox.core.endpoint import Endpoint
from pynetbox.core.query import Request
from pynetbox.models import dcim, ipam, virtualization, circuits


class App(object):
    """ Represents apps in NetBox.

    Calls to attributes are returned as Endpoint objects.

    :returns: :py:class:`.Endpoint` matching requested attribute.
    :raises: :py:class:`.RequestError`
        if requested endpoint doesn't exist.
    """

    def __init__(self, api, name, model=None):
        self.api = api
        self.name = name
        self.model = model
        self._choices = None

    def __getattr__(self, name):
        return Endpoint(self.api, self, name, model=self.model)

    def choices(self):
        """ Returns _choices response from App

        :Returns: Raw response from NetBox's _choices endpoint.
        """
        if self._choices:
            return self._choices

        self._choices = Request(
            base="{}/{}/_choices/".format(self.api.base_url, self.name),
            ssl_verify=self.api.ssl_verify,
        ).get()

        return self._choices


class Api(object):
    """ The API object is the point of entry to pynetbox.

    After instantiating the Api() with the appropriate named arguments
    you can specify which app and endpoint you wish to interact with.

    Valid attributes currently are:
        * dcim
        * ipam
        * circuits
        * secrets
        * tenancy
        * extras
        * virtualization

    Calling any of these attributes will return
    :py:class:`.App` which exposes endpoints as attributes.

    :type ssl_verify: bool or str
    :param str url: The base url to the instance of Netbox you
        wish to connect to.
    :param str token: Your netbox token.
    :param str,optional private_key_file: The path to your private
        key file.
    :param str,optional private_key: Your private key.
    :param str,optional version: Override the API version, otherwise
        it's dynamically discovered.
    :param bool/str,optional ssl_verify: Specify SSL verification behavior
        see: requests_.
    :raises ValueError: If *private_key* and *private_key_file* are both
        specified.
    :raises AttributeError: If app doesn't exist.
    :Examples:

    >>> import pynetbox
    >>> nb = pynetbox.api(
    ...     'http://localhost:8000',
    ...     private_key_file='/path/to/private-key.pem',
    ...     token='d6f4e314a5b5fefd164995169f28ae32d987704f'
    ... )
    >>> nb.dcim.devices.all()

    .. _requests: http://docs.python-requests.org/en/master/user/advanced/#ssl-cert-verification
    """  # noqa

    def __init__(
        self,
        url,
        token=None,
        private_key=None,
        private_key_file=None,
        ssl_verify=True,
    ):
        if private_key and private_key_file:
            raise ValueError(
                '"private_key" and "private_key_file" cannot be used together.'
            )
        base_url = "{}/api".format(url if url[-1] != "/" else url[:-1])
        self.token = token
        self.private_key = private_key
        self.private_key_file = private_key_file
        self.base_url = base_url
        self.ssl_verify = ssl_verify
        self.session_key = None

        if self.private_key_file:
            with open(self.private_key_file, "r") as kf:
                private_key = kf.read()
                self.private_key = private_key

        req = Request(
            base=base_url,
            token=token,
            private_key=private_key,
            ssl_verify=ssl_verify,
        )
        if self.token and self.private_key:
            self.session_key = req.get_session_key()

        self.dcim = App(self, "dcim", model=dcim)
        self.ipam = App(self, "ipam", model=ipam)
        self.circuits = App(self, "circuits", model=circuits)
        self.secrets = App(self, "secrets", model=None)
        self.tenancy = App(self, "tenancy", model=None)
        self.extras = App(self, "extras", model=None)
        self.virtualization = App(self, "virtualization", model=virtualization)



import unittest

from pynetbox.core.response import Record


class RecordTestCase(unittest.TestCase):

    def test_serialize_list_of_records(self):
        test_values = {
            'id': 123,
            "tagged_vlans": [
                {
                    "id": 1,
                    "url": "http://localhost:8000/api/ipam/vlans/1/",
                    "vid": 1,
                    "name": "test1",
                    "display_name": "test1"
                },
                {
                    "id": 2,
                    "url": "http://localhost:8000/api/ipam/vlans/2/",
                    "vid": 2,
                    "name": "test 2",
                    "display_name": "test2"
                }
            ],
        }
        test_obj = Record(test_values, None, None)
        test = test_obj.serialize()
        self.assertEqual(test['tagged_vlans'], [1, 2])

    def test_serialize_list_of_ints(self):
        test_values = {
            'id': 123,
            'units': [12],
        }
        test_obj = Record(test_values, None, None)
        test = test_obj.serialize()
        self.assertEqual(test['units'], [12])

    def test_serialize_tag_set(self):
        test_values = {
            'id': 123,
            'tags': [
                'foo',
                'bar',
                'foo',
            ],
        }
        test = Record(test_values, None, None).serialize()
        self.assertEqual(len(test['tags']), 2)

    def test_diff(self):
        test_values = {
            'id': 123,
            'custom_fields': {
                'foo': 'bar'
            },
            'string_field': 'foobar',
            'int_field': 1,
            "nested_dict": {
                "id": 222,
                "name": 'bar',
            },
            'tags': [
                'foo',
                'bar',
            ]
        }
        test = Record(test_values, None, None)
        test.tags.append('baz')
        test.nested_dict = 1
        test.string_field = 'foobaz'
        self.assertEqual(test._diff(), {'tags', 'nested_dict', 'string_field'})



import unittest
import six

import pynetbox
from .util import Response

if six.PY3:
    from unittest.mock import patch
else:
    from mock import patch

host = "http://localhost:8000"

def_kwargs = {
    'token': 'abc123',
    'private_key_file': 'tests/fixtures/api/get_session_key.json',
}

# Keys are app names, values are arbitrarily selected endpoints
# We use dcim and ipam since they have unique app classes
# and circuits because it does not. We don't add other apps/endpoints
# beyond 'circuits' as they all use the same code as each other
endpoints = {
    'dcim': 'devices',
    'ipam': 'prefixes',
    'circuits': 'circuits',
}


class ApiTestCase(unittest.TestCase):

    @patch(
        'pynetbox.core.query.requests.post',
        return_value=Response(fixture='api/get_session_key.json')
    )
    def test_get(self, *_):
        api = pynetbox.api(
            host,
            **def_kwargs
        )
        self.assertTrue(api)

    @patch(
        'pynetbox.core.query.requests.post',
        return_value=Response(fixture='api/get_session_key.json')
    )
    def test_sanitize_url(self, *_):
        api = pynetbox.api(
            'http://localhost:8000/',
            **def_kwargs
        )
        self.assertTrue(api)
        self.assertEqual(api.base_url, 'http://localhost:8000/api')


class ApiArgumentsTestCase(unittest.TestCase):

    @patch(
        'pynetbox.core.query.requests.post',
        return_value=Response(fixture='api/get_session_key.json')
    )
    def common_arguments(self, kwargs, arg, expect, *_):
        '''
        Ensures the api and endpoint instances have ssl_verify set
        as expected
        '''
        api = pynetbox.api(
            host,
            **kwargs
        )
        self.assertIs(getattr(api, arg, "fail"), expect)
        for app, endpoint in endpoints.items():
            ep = getattr(getattr(api, app), endpoint)
            self.assertIs(getattr(ep, arg), expect)

    def test_ssl_verify_default(self):
        self.common_arguments(def_kwargs, 'ssl_verify', True)

    def test_ssl_verify_true(self):
        kwargs = dict(def_kwargs, **{'ssl_verify': True})
        self.common_arguments(kwargs, 'ssl_verify', True)

    def test_ssl_verify_false(self):
        kwargs = dict(def_kwargs, **{'ssl_verify': False})
        self.common_arguments(kwargs, 'ssl_verify', False)

    def test_ssl_verify_string(self):
        kwargs = dict(def_kwargs, **{'ssl_verify': '/path/to/bundle'})
        self.common_arguments(kwargs, 'ssl_verify', '/path/to/bundle')



import unittest

import six

from pynetbox.core.endpoint import Endpoint

if six.PY3:
    from unittest.mock import patch, Mock
else:
    from mock import patch, Mock


class EndPointTestCase(unittest.TestCase):

    def test_filter(self):
        with patch(
            "pynetbox.core.query.Request.get", return_value=Mock()
        ) as mock:
            api = Mock(base_url="http://localhost:8000/api")
            app = Mock(name="test")
            mock.return_value = [{'id': 123}, {'id': 321}]
            test_obj = Endpoint(api, app, "test")
            test = test_obj.filter(test="test")
            self.assertEqual(len(test), 2)

    def test_filter_empty_kwargs(self):

        api = Mock(base_url="http://localhost:8000/api")
        app = Mock(name="test")
        test_obj = Endpoint(api, app, "test")
        with self.assertRaises(ValueError) as _:
            test_obj.filter()

    def test_filter_reserved_kwargs(self):

        api = Mock(base_url="http://localhost:8000/api")
        app = Mock(name="test")
        test_obj = Endpoint(api, app, "test")
        with self.assertRaises(ValueError) as _:
            test_obj.filter(id=1)



"""
(c) 2017 DigitalOcean

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from pynetbox.core.response import Record
from pynetbox.core.endpoint import RODetailEndpoint
from pynetbox.models.ipam import IpAddresses
from pynetbox.models.circuits import Circuits


class DeviceTypes(Record):
    def __str__(self):
        return self.model


class Devices(Record):
    """Devices Object

        Represents a device response from netbox.

        Attributes:
            primary_ip, ip4, ip6 (list): Tells __init__ in Record() to
                take the `primary_ip` field's value from the API
                response and return an initialized list of IpAddress
                objects
            device_type (obj): Tells __init__ in Record() to take the
                `device_type` field's value from the API response and
                return an initialized DeviceType object
    """

    has_details = True
    device_type = DeviceTypes
    primary_ip = IpAddresses
    primary_ip4 = IpAddresses
    primary_ip6 = IpAddresses

    @property
    def napalm(self):
        """ Represents the ``napalm`` detail endpoint.

        Returns a DetailEndpoint object that is the interface for
        viewing response from the napalm endpoint.

        :returns: :py:class:`.DetailEndpoint`

        :Examples:

        >>> device = nb.ipam.devices.get(123)
        >>> device.napalm.list(method='get_facts')
        {"get_facts": {"interface_list": ["ge-0/0/0"]}}

        """
        return RODetailEndpoint(self, "napalm")


class InterfaceConnections(Record):
    def __str__(self):
        return self.interface_a.name


class InterfaceConnection(Record):
    def __str__(self):
        return self.interface.name


class ConnectedEndpoint(Record):
    device = Devices


class Interfaces(Record):
    interface_connection = InterfaceConnection
    connected_endpoint = ConnectedEndpoint


class RackReservations(Record):
    def __str__(self):
        return self.description


class VirtualChassis(Record):
    def __str__(self):
        return self.master.display_name


class RUs(Record):

    device = Devices


class Racks(Record):
    @property
    def units(self):
        """ Represents the ``units`` detail endpoint.

        Returns a DetailEndpoint object that is the interface for
        viewing response from the units endpoint.

        :returns: :py:class:`.DetailEndpoint`

        :Examples:

        >>> rack = nb.dcim.racks.get(123)
        >>> rack.units.list()
        {"get_facts": {"interface_list": ["ge-0/0/0"]}}

        """
        return RODetailEndpoint(self, "units", custom_return=RUs)


class Termination(Record):
    def __str__(self):
        # hacky check to see if we're a circuit termination to
        # avoid another call to NetBox because of a non-existent attr
        # in self.name
        if "circuit" in str(self.url):
            return self.circuit.cid

        return self.name

    device = Devices
    circuit = Circuits


class Cables(Record):
    def __str__(self):
        return "{} <> {}".format(self.termination_a, self.termination_b)

    termination_a = Termination
    termination_b = Termination



import unittest
import six

from .util import Response
import pynetbox

if six.PY3:
    from unittest.mock import patch
else:
    from mock import patch

api = pynetbox.api(
    "http://localhost:8000",
)

nb = api.circuits

HEADERS = {
    'accept': 'application/json;'
}


class Generic(object):
    class Tests(unittest.TestCase):
        name = ''
        ret = pynetbox.core.response.Record
        app = 'circuits'

        def test_get_all(self):
            with patch(
                'pynetbox.core.query.requests.get',
                return_value=Response(fixture='{}/{}.json'.format(
                    self.app,
                    self.name
                ))
            ) as mock:
                ret = getattr(nb, self.name).all()
                self.assertTrue(ret)
                self.assertTrue(isinstance(ret, list))
                self.assertTrue(isinstance(ret[0], self.ret))
                mock.assert_called_with(
                    'http://localhost:8000/api/{}/{}/'.format(
                        self.app,
                        self.name.replace('_', '-')
                    ),
                    headers=HEADERS,
                    verify=True
                )

        def test_filter(self):
            with patch(
                'pynetbox.core.query.requests.get',
                return_value=Response(fixture='{}/{}.json'.format(
                    self.app,
                    self.name
                ))
            ) as mock:
                ret = getattr(nb, self.name).filter(name='test')
                self.assertTrue(ret)
                self.assertTrue(isinstance(ret, list))
                self.assertTrue(isinstance(ret[0], self.ret))
                mock.assert_called_with(
                    'http://localhost:8000/api/{}/{}/?name=test'.format(
                        self.app,
                        self.name.replace('_', '-')
                    ),
                    headers=HEADERS,
                    verify=True
                )

        def test_get(self):
            with patch(
                'pynetbox.core.query.requests.get',
                return_value=Response(fixture='{}/{}.json'.format(
                    self.app,
                    self.name[:-1]
                ))
            ) as mock:
                ret = getattr(nb, self.name).get(1)
                self.assertTrue(ret)
                self.assertTrue(isinstance(ret, self.ret))
                mock.assert_called_with(
                    'http://localhost:8000/api/{}/{}/1/'.format(
                        self.app,
                        self.name.replace('_', '-')
                    ),
                    headers=HEADERS,
                    verify=True
                )


class CircuitsTestCase(Generic.Tests):
    name = 'circuits'

    @patch(
        'pynetbox.core.query.requests.get',
        return_value=Response(fixture='circuits/circuit.json')
    )
    def test_repr(self, _):
        test = nb.circuits.get(1)
        self.assertEqual(str(test), '123456')


class ProviderTestCase(Generic.Tests):
    name = 'providers'


class CircuitTypeTestCase(Generic.Tests):
    name = 'circuit_types'


class CircuitTerminationsTestCase(Generic.Tests):
    name = 'circuit_terminations'

    @patch(
        'pynetbox.core.query.requests.get',
        return_value=Response(fixture='circuits/circuit_termination.json')
    )
    def test_repr(self, _):
        test = nb.circuit_terminations.get(1)
        self.assertEqual(str(test), '123456')
